/*
 * Astfin Team  - Zarlink Echo Cancellation Module
 *
 *
 * Copyright (C) 2008, Astfin / uCpbx Ltd.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
 *
 */


#ifndef EC_ZLXXXXX_H
#define EC_ZLXXXXX_H

#define BF_ASYNMEMBASE  0x20000000
#define BF_ASYNMEMSIZE  0x00100000
#define BF_ASYNBANKNUM  0x2
#define EC_BASE_ADDR	(BF_ASYNMEMBASE+(BF_ASYNMEMSIZE*BF_ASYNBANKNUM))

#define GPIO_EC_PRESENT 	GPIO_PG1	//* pg1 // Low if e/c module present
#define GPIO_EC_32	 	GPIO_PG2	//* pg2 // High for zl38065/zl50232 and Low if zl50234/zl50235 series

//****************************************************************************
//* Types definitions.
typedef volatile unsigned char * regA_t;	//*** Define register address type...

//* Define state values
typedef enum {
	EC_NA = -1,
	EC_FREE,
	EC_BUSY
} EC_STAT;

typedef enum {
	SUCCESS,
	FAILURE
} ECRETCODE_t;

typedef enum {
	LINEAR,
	ULAW,
	ALAW
} ECLAW_t;

typedef unsigned short USHORT_t;

//* Include both files since both EC may be used based on configuration...
//* In the case of use of ZL38065 allways include "ec_zl38065.h" FIRST !!
#include "ec_zl38065.h"
#include "ec_zl50235.h"

//****************************************************************************
//* Function prototypes for exported functions.
ECRETCODE_t zl38065_echocan(int channel, int tail_ms);
ECRETCODE_t echocan_tail_length(int channel, int tail_ms);
ECRETCODE_t echocan_setEnc(ECLAW_t code);
ECRETCODE_t echocan_setGains(int channel, int gainRin, int gainRout,
	int gainSin, int gainSout);
ECRETCODE_t echocan_set_uProfile(int channel, USHORT_t iFlatDelay,
	USHORT_t iDecayStepSize, USHORT_t iDecayStepNum);
//* Define a make address macro
#define	ECREGADDR(x)	(EC_BASE_ADDR + ((x)<<1))

//* Define commands to ioctl()
#define	ECCMD_NOP		0

#endif
